@@ -6,7 +6,8 @@ BiDiMailUI.Composition = {
   alternativeEnterBehavior : null,
     // The default behavior of the Enter key in HTML mail messages
     // is to insert a <br>; the alternative behavior we implement
-    // is to close a paragraph and begin a new one
+    // is to close a paragraph and begin a new one (assuming we're
+    // in Paragraph Mode)
 
   CtrlShiftMachine : {
     // We have implemented a Mealy automaton for implementing the Ctrl+Shift
@@ -99,8 +100,16 @@ BiDiMailUI.Composition = {
           }} );
 
         BiDiMailUI.Composition.setParagraphMarginsRule();
-        if (BiDiMailUI.Composition.alternativeEnterBehavior)
-          BiDiMailUI.Composition.setParagraphMode();
+        // note that the "alternative Enter key behavior" is only
+        // relevant to paragraph mode; we used to always try to set
+        // paragraph mode to express that behavior, but several users
+        // have been complaining...
+        var startCompositionInParagraphMode =
+          BiDiMailUI.Prefs.getBoolPref("compose.start_composition_in_paragraph_mode", false);
+        if (startCompositionInParagraphMode)
+          BiDiMailUI.Composition.setParagraphMode("p");
+        else 
+          BiDiMailUI.Composition.setParagraphMode("");
       }
 
       BiDiMailUI.Composition.setInitialDirection(this.messageParams);
@@ -423,7 +432,8 @@ BiDiMailUI.Composition = {
 
     editor.beginTransaction();
 
-    var css = document.createElement('style');
+    var css = 
+      document.getElementById("content-frame").contentDocument.createElement('style');
     css.type = 'text/css';
     if (css.styleSheet) {
       css.styleSheet.cssText = styleRulesText;
@@ -469,10 +479,11 @@ BiDiMailUI.Composition = {
   },
 
   // Our extension likes "Paragraph Mode" rather than "Body Text" mode
-  // for composing messages, - since paragraph are block elements, with
-  // a direction setting
+  // for composing messages - since paragraph are block elements, with
+  // a direction setting; this function can set the mode either way,
+  // using "p" or "" (for paragraph and body respectively)
 
-  setParagraphMode : function() {
+  setParagraphMode : function(modeStr) {
     var editor = GetCurrentEditor();
     if (!editor) {
 #ifdef DEBUG_handleComposeReplyCSS
@@ -481,10 +492,10 @@ BiDiMailUI.Composition = {
       dump('setParagraphMode failed to acquire editor object.');
       return;
     }
-    editor.setParagraphFormat("p");
+    editor.setParagraphFormat(modeStr);
     // as we don't use doStatefulCommand, we need to update the command
     // state attribute...
-    document.getElementById("cmd_paragraphState").setAttribute("state", "p");
+    document.getElementById("cmd_paragraphState").setAttribute("state", modeStr);
   },
 
   getDisplayedCopyParams : function(messageURI,messageParams) {
@@ -773,7 +784,7 @@ BiDiMailUI.Composition = {
     // necessary on some systems to get past the point when the direction
     // is set to LTR (not by us)
     BiDiMailUI.Composition.bodyReadyListener.workaroundForcingTimeoutId = 
-      setTimeout("BiDiMailUI.Composition.bodyReadyListener.NotifyComposeBodyReady()", 200);
+      setTimeout("BiDiMailUI.Composition.bodyReadyListener.NotifyComposeBodyReady()", 400);
 
 
 #ifdef DEBUG_composeWindowOnActualLoad
@@ -802,7 +813,8 @@ BiDiMailUI.Composition = {
       BiDiMailUI.Composition.alternativeEnterBehavior =
         BiDiMailUI.Prefs.getBoolPref("compose.alternative_enter_behavior", true);
       // Applying the alternative Enter behavior requires the editor to be
-      // in paragraph mode; but we won't set it until the body is ready
+      // in paragraph mode; but we won't consider doing that until the body is
+      // ready.
     }
 
     BiDiMailUI.Composition.directionSwitchController.setAllCasters();
@@ -855,6 +867,10 @@ BiDiMailUI.Composition = {
     }
     else {
       dump("gMsgCompose not ready for this message in BiDiMailUI.Composition.composeWindowOnReopen()");
+#ifdef DEBUG_ComposeEvents
+      BiDiMailUI.JSConsoleService.logStringMessage(
+        "gMsgCompose not ready for this message in BiDiMailUI.Composition.composeWindowOnReopen()");
+#endif
     }
   },
 
@@ -1134,7 +1150,7 @@ BiDiMailUI.Composition = {
 #ifdef DEBUG_keypress
     BiDiMailUI.JSConsoleService.logStringMessage('SWITCHING paragraph');
 #endif
-    SwitchParagraphDirection();
+    BiDiMailUI.Composition.SwitchParagraphDirection();
         }
         else {
 #ifdef DEBUG_keypress
@@ -1214,8 +1230,9 @@ BiDiMailUI.Composition = {
       var isParMixed = { value: false }; // would be ignored
       var parState = editor.getParagraphState(isParMixed);
 
-      // we currently apply our own enter behavior to
-      // paragraph states "p" and "h1" through "h6"
+      // We currently apply our own enter behavior to
+      // paragraph states "p" and "h1" through "h6"; specifically,
+      // we don't apply it in Body Text mode 
 
       if (parState != "p" && parState.length != 2 )
         return;
