@@ -1,7 +1,8 @@
 // Code outside BiDi Mail UI should only use the 
 // BiDiMailUI.Display.ActionPhases and perhaps the
-// BiDiMailUI.Display.setMessageDirectionForcing function: ActionPhase functions
-// are the four phases of action performed when loading a message
+// BiDiMailUI.Display.setMessageDirectionForcing function
+// (ActionPhase functions are the four phases of action 
+// performed when loading a message)
 
 BiDiMailUI.Display = {
   ActionPhases : {
@@ -171,7 +172,9 @@ BiDiMailUI.Display = {
       BiDiMailUI.JSConsoleService.logStringMessage("-----\ntext node\n-----\n" + node.nodeValue);
 #endif
       // TODO: ensure the parent's a PRE or BLOCKQUOTE or something else that's nice
-      if (! /\n[ \f\r\t\v\n\u00A0\\u2028\\u2029!-@\[-`{-\xA0\u2013\u2014\uFFFD]*\n/m.test(node.nodeValue)) {
+      textSplit = new RegExp (BiDiMailUI.RegExpStrings.TEXT_SPLIT_SEQUENCE, "m");
+
+      if (! textSplit.test(node.nodeValue)) {
          node = treeWalker.nextNode();
          continue;
       }
@@ -521,7 +524,8 @@ BiDiMailUI.Display = {
          U = UTF-8, 
        we won't handle any issues with other charsets
     3. Message contains windows-1255/6 text (Y/N)
-    4. Message contains UTF-8 text (Y/N)
+    4. Message contains UTF-8 text (that is, UTF-8 octet sequences 
+       which are not iso-8859-1 chars 0-127) (Y/N)
 
     What should we do for each combination of values? 
     (* means all possible values)
@@ -544,9 +548,9 @@ BiDiMailUI.Display = {
     YNNY - recode UTF-8 text
     YNYN - recode windows-1255/6 text
     YC*Y - This is very bad, since we're not allowed to change charset;
-           we'll try recording UTF-8 text, but it'll probably won't work well
+           we'll try recoding UTF-8 text, but it probably won't work well
     *UY* - This is very bad, since we're not allowed to change charset;
-           we'll try recording windows-1255/6 text, but it'll probably won't work well
+           we'll try recoding windows-1255/6 text, but it probably won't work well
            
     Extra Notes:
 
@@ -621,7 +625,7 @@ BiDiMailUI.Display = {
         // need to look for a character in the Hebrew or Arabic Unicode range
         contentToMatch = new RegExp(
           (cMCParams.mailnewsDecodingType == "latin-charset") ?
-          // Here we want a sequence of Unicode value of characters whose 
+          // Here we want a sequence of Unicode values of characters whose 
           // windows-1252 octet is such that would be decoded as 'clearly'
           // Hebrew or Arabic text; we could be less or more picky depending
           // on what we feel about characters like power-of-2, paragraph-mark,
